@@ -21,13 +21,20 @@
 #include "config.h"
 #include "trampoline.h"
 
-#if defined(__hppa__)
+#if defined(__hppa__) && !defined(__hppa64__)
 #if 0
 #define __hppaold__  /* Old trampoline, real machine code. */
 #else
 #define __hppanew__  /* New trampoline, just a closure. */
 #endif
 #endif
+#if defined(__hppa64__)
+#if 0
+#define __hppa64old__  /* Old trampoline, real machine code. */
+#else
+#define __hppa64new__  /* New trampoline, just a closure. */
+#endif
+#endif
 #if defined(__powerpc__) && !defined(__powerpc64__)
 #if !defined(_AIX)
 #define __powerpcsysv4__  /* SysV.4 ABI, real machine code. */
@@ -38,10 +45,10 @@
 #if defined(__powerpc64__) && !defined(__powerpc64_elfv2__)
 #define __powerpc64aix__  /* AIX ABI, just a closure. */
 #endif
-#if defined(__hppanew__)
+#if defined(__hppanew__) || defined(__hppa64new__)
 /*
  * A function pointer is a biased pointer to a data area whose first word
- * contains the actual address of the function.
+ * (hppa) or third word (hppa64) contains the actual address of the function.
  */
 extern void tramp (); /* trampoline prototype */
 /* We don't need to take any special measures to make the code executable
@@ -88,7 +95,11 @@ extern void (*tramp) (); /* trampoline prototype */
         /* Use an shmat'ed page. */
         #define EXECUTABLE_VIA_SHM
       #else
-        ??
+        #if (defined(_WIN32) || defined(__WIN32__)) && ! defined(__CYGWIN__) /* native Windows */
+          #define EXECUTABLE_VIA_VIRTUALALLOC
+        #else
+          ??
+        #endif
       #endif
     #endif
   #endif
@@ -102,6 +113,9 @@ extern void (*tramp) (); /* trampoline prototype */
 #include <unistd.h>
 #endif
 
+/* Define intptr_t, uintptr_t. */
+#include <stdint.h>
+
 /* Declare getpagesize(). */
 #ifdef HAVE_GETPAGESIZE
 #ifdef __cplusplus
@@ -124,9 +138,6 @@ extern RETGETPAGESIZETYPE getpagesize (void);
 #ifdef HAVE_MACH_VM
 #include <sys/resource.h>
 #include <mach/mach_interface.h>
-#ifdef NeXT
-#include <mach/mach_init.h>
-#endif
 #ifdef __osf__
 #include <mach_init.h>
 #endif
@@ -163,6 +174,13 @@ extern RETGETPAGESIZETYPE getpagesize (void);
 #endif
 #endif
 
+/* Declare VirtualAlloc(), GetSystemInfo. */
+#ifdef EXECUTABLE_VIA_VIRTUALALLOC
+#define WIN32_LEAN_AND_MEAN
+#define WIN32_EXTRA_LEAN
+#include <windows.h>
+#endif
+
 /* Some old mmap() implementations require the flag MAP_FILE whenever you pass
    an fd >= 0. */
 #ifndef MAP_FILE
@@ -176,27 +194,23 @@ extern RETGETPAGESIZETYPE getpagesize (void);
 
 /* Support for instruction cache flush. */
 #ifdef __i386__
-#if defined(_WIN32) /* WindowsNT or Windows95 */
+#if (defined(_WIN32) || defined(__WIN32__)) && ! defined(__CYGWIN__) /* native Windows */
 #define WIN32_LEAN_AND_MEAN
 #define WIN32_EXTRA_LEAN
 #include <windows.h>
 #endif
 #endif
 #if defined(__mips__) || defined(__mipsn32__) || defined(__mips64__)
-#ifdef linux
-#include <asm/cachectl.h>
+#ifdef HAVE_SYS_CACHECTL_H /* IRIX, Linux */
+#include <sys/cachectl.h>
 #else
 #ifdef __OpenBSD__
 #include <machine/sysarch.h>
-#else
-#ifdef HAVE_SYS_CACHECTL_H
-#include <sys/cachectl.h>
-#endif
 #endif
 #endif
 #endif
 /* Inline assembly function for instruction cache flush. */
-#if defined(__sparc__) || defined(__sparc64__) || defined(__alpha__) || defined(__hppaold__) || defined(__powerpcsysv4__) || defined(__powerpc64_elfv2__)
+#if defined(__sparc__) || defined(__sparc64__) || defined(__alpha__) || defined(__hppaold__) || defined(__hppa64old__) || defined(__powerpcsysv4__) || defined(__powerpc64_elfv2__)
 #if defined(__sparc__) || defined(__sparc64__)
 extern void __TR_clear_cache_4();
 #else
@@ -274,6 +288,15 @@ static int open_noinherit (const char *filename, int flags, int mode)
 #define TRAMP_ALIGN 16
 #define TRAMP_BIAS 2
 #endif
+#ifdef __hppa64old__
+#define TRAMP_LENGTH 96
+#define TRAMP_ALIGN 8
+#define TRAMP_BIAS 64
+#endif
+#ifdef __hppa64new__
+#define TRAMP_LENGTH 56
+#define TRAMP_ALIGN 8
+#endif
 #if defined(__arm__) || defined(__armhf__)
 #define TRAMP_LENGTH 36
 #define TRAMP_ALIGN 4
@@ -378,7 +401,7 @@ static char* freelist = NULL;
 gl_lock_define_initialized(static, freelist_lock)
 #endif
 
-__TR_function alloc_trampoline (__TR_function address, void* variable, void* data)
+trampoline_function_t alloc_trampoline (trampoline_function_t address, void** variable, void* data)
 {
   char* function;
   char* function_x;
@@ -389,7 +412,14 @@ __TR_function alloc_trampoline (__TR_function address, void* variable, void* dat
     {
       /* Simultaneous execution of this initialization in multiple threads
          is OK. */
-#if defined(HAVE_MACH_VM)
+#if defined(EXECUTABLE_VIA_VIRTUALALLOC)
+      /* GetSystemInfo
+         <https://msdn.microsoft.com/en-us/library/ms724381.aspx>
+         <https://msdn.microsoft.com/en-us/library/ms724958.aspx> */
+      SYSTEM_INFO info;
+      GetSystemInfo(&info);
+      pagesize = info.dwPageSize;
+#elif defined(HAVE_MACH_VM)
       pagesize = vm_page_size;
 #else
       pagesize = getpagesize();
@@ -410,6 +440,15 @@ __TR_function alloc_trampoline (__TR_function address, void* variable, void* dat
     { /* Get a new page. */
       char* page;
       char* page_end;
+#ifdef EXECUTABLE_VIA_VIRTUALALLOC
+      /* VirtualAlloc
+         <https://msdn.microsoft.com/en-us/library/aa366887.aspx>
+         <https://msdn.microsoft.com/en-us/library/aa366786.aspx> */
+      page = VirtualAlloc(NULL,pagesize,MEM_COMMIT,PAGE_EXECUTE_READWRITE);
+      if (page == NULL)
+        { fprintf(stderr,"trampoline: Out of virtual memory!\n"); abort(); }
+      page_end = page + pagesize;
+#else
 #ifdef EXECUTABLE_VIA_MMAP_FILE_SHARED
       char* page_x;
       /* Extend the file by one page. */
@@ -424,8 +463,8 @@ __TR_function alloc_trampoline (__TR_function address, void* variable, void* dat
       file_length = new_file_length;
       page_end = page + pagesize;
       /* Link the two pages together. */
-      ((long*)page)[0] = page_x - page;
-      page = (char*)(((long)page + sizeof(long) + TRAMP_ALIGN-1) & -TRAMP_ALIGN);
+      ((intptr_t*)page)[0] = page_x - page;
+      page = (char*)(((uintptr_t)page + sizeof(intptr_t) + TRAMP_ALIGN-1) & -TRAMP_ALIGN);
 #else
 #ifdef EXECUTABLE_VIA_MMAP_THEN_MPROTECT
 #ifdef HAVE_MMAP_ANONYMOUS
@@ -455,12 +494,13 @@ __TR_function alloc_trampoline (__TR_function address, void* variable, void* dat
       if (page == (char*)(-1))
         { fprintf(stderr,"trampoline: Out of virtual memory!\n"); abort(); }
       page_end = page + pagesize;
+#endif
 #endif
       /* Fill it with free trampolines. */
       { char** last = &freelist;
         while (page+TRAMP_LENGTH <= page_end)
           { *last = page; last = (char**)page;
-            page = (char*)(((long)page + TRAMP_LENGTH + TRAMP_ALIGN-1) & -TRAMP_ALIGN);
+            page = (char*)(((uintptr_t)page + TRAMP_LENGTH + TRAMP_ALIGN-1) & -TRAMP_ALIGN);
           }
         *last = NULL;
     } }
@@ -470,15 +510,15 @@ __TR_function alloc_trampoline (__TR_function address, void* variable, void* dat
   { char* room = (char*) malloc(sizeof(void*) + TRAMP_LENGTH + TRAMP_ALIGN-1);
     if (!room)
       { fprintf(stderr,"trampoline: Out of virtual memory!\n"); abort(); }
-    function = (char*)(((long)room + sizeof(void*) + TRAMP_ALIGN-1) & -TRAMP_ALIGN);
+    function = (char*)(((uintptr_t)room + sizeof(void*) + TRAMP_ALIGN-1) & -TRAMP_ALIGN);
     ((char**)function)[-1] = room; /* backpointer for free_trampoline() */
   }
 #endif
 
 #if !defined(CODE_EXECUTABLE) && defined(EXECUTABLE_VIA_MMAP_FILE_SHARED)
   /* Find the executable address corresponding to the writable address. */
-  { unsigned long page = (unsigned long) function & -(long)pagesize;
-    function_x = function + ((long*)page)[0];
+  { uintptr_t page = (uintptr_t) function & -(intptr_t)pagesize;
+    function_x = function + ((intptr_t*)page)[0];
   }
 #else
   function_x = function;
@@ -951,6 +991,90 @@ __TR_function alloc_trampoline (__TR_function address, void* variable, void* dat
 #define tramp_data(function)  \
   ((long *) function)[3]
 #endif
+#ifdef __hppa64old__
+  /* function:
+   *    mfia %r27			000014BB
+   *    ldd 40(%r27),%r31		537F0050
+   *    ldd 48(%r27),%r1		53610060
+   *    std %r1,0(%r31)			0FE112C0
+   *    ldd 56(%r27),%r27		537B0070
+   *    ldd 16(%r27),%r1		53610020
+   *    ldd 24(%r27),%r27		537B0030
+   *    bve (%r1)			E820D000
+   *     nop				08000240
+   *    .align 8
+   *    .dword <variable>
+   *    .dword <data>
+   *    .dword <address>
+   * function_pointer:
+   *    .dword 0
+   *    .dword 0
+   *    .dword function
+   *    .dword 0
+   */
+  *(int *)  (function + 0) = 0x000014BB;
+  *(int *)  (function + 4) = 0x537F0050;
+  *(int *)  (function + 8) = 0x53610060;
+  *(int *)  (function +12) = 0x0FE112C0;
+  *(int *)  (function +16) = 0x537B0070;
+  *(int *)  (function +20) = 0x53610020;
+  *(int *)  (function +24) = 0x537B0030;
+  *(int *)  (function +28) = 0xE820D000;
+  *(int *)  (function +32) = 0x08000240;
+  *(long *) (function +40) = (long)variable;
+  *(long *) (function +48) = (long)data;
+  *(long *) (function +56) = (long)address;
+  *(long *) (function +64) = (long)0;
+  *(long *) (function +72) = (long)0;
+  *(long *) (function +80) = (long)function;
+  *(long *) (function +88) = (long)0;
+#define TRAMP_CODE_LENGTH  36
+#define is_tramp(function)  \
+  *(int *) (function + 0) == 0x000014BB && \
+  *(int *) (function + 4) == 0x537F0050 && \
+  *(int *) (function + 8) == 0x53610060 && \
+  *(int *) (function +12) == 0x0FE112C0 && \
+  *(int *) (function +16) == 0x537B0070 && \
+  *(int *) (function +20) == 0x53610020 && \
+  *(int *) (function +24) == 0x537B0030 && \
+  *(int *) (function +28) == 0xE820D000 && \
+  *(int *) (function +32) == 0x08000240
+#define tramp_address(function)  \
+  (*(unsigned long *) (function +56))
+#define tramp_variable(function)  \
+  (*(unsigned long *) (function +40))
+#define tramp_data(function)  \
+  (*(unsigned long *) (function +48))
+#endif
+#ifdef __hppa64new__
+  /* function:
+   *    .dword 0
+   *    .dword 0
+   *    .dword tramp
+   *    .dword closure
+   * closure:
+   *    .dword <variable>
+   *    .dword <data>
+   *    .dword <address>
+   */
+  *(long *) (function + 0) = 0;
+  *(long *) (function + 8) = 0;
+  *(long *) (function +16) = ((long *) (void*) &tramp)[2];
+  *(long *) (function +24) = (long) (function + 32);
+  *(long *) (function +32) = (long) variable;
+  *(long *) (function +40) = (long) data;
+  *(long *) (function +48) = (long) address;
+#define TRAMP_CODE_LENGTH  0
+#define is_tramp(function)  \
+  ((long *) function)[2] == ((long *) (void*) &tramp)[2] && \
+  ((long *) function)[3] == (long) (function + 32)
+#define tramp_address(function)  \
+  ((long *) function)[6]
+#define tramp_variable(function)  \
+  ((long *) function)[4]
+#define tramp_data(function)  \
+  ((long *) function)[5]
+#endif
 #if defined(__arm__) || defined(__armhf__)
   /* function:
    *	stmfd	sp!,{r0}		E92D0001
@@ -1218,16 +1342,16 @@ __TR_function alloc_trampoline (__TR_function address, void* variable, void* dat
    *    jmp *%rax			FF E0
    */
   *(short *) (function + 0) = 0xB848;
-  *(short *) (function + 2) = (unsigned long) data & 0xffff;
-  *(int *)   (function + 4) = ((unsigned long) data >> 16) & 0xffffffff;
-  *(short *) (function + 8) = ((unsigned long) data >> 48) & 0xffff;
+  *(short *) (function + 2) = (unsigned long long) data & 0xffff;
+  *(int *)   (function + 4) = ((unsigned long long) data >> 16) & 0xffffffff;
+  *(short *) (function + 8) = ((unsigned long long) data >> 48) & 0xffff;
   *(short *) (function +10) = 0xA348;
-  *(int *)   (function +12) = (unsigned long) variable & 0xffffffff;
-  *(int *)   (function +16) = ((unsigned long) variable >> 32) & 0xffffffff;
+  *(int *)   (function +12) = (unsigned long long) variable & 0xffffffff;
+  *(int *)   (function +16) = ((unsigned long long) variable >> 32) & 0xffffffff;
   *(short *) (function +20) = 0xB848;
-  *(short *) (function +22) = (unsigned long) address & 0xffff;
-  *(int *)   (function +24) = ((unsigned long) address >> 16) & 0xffffffff;
-  *(short *) (function +28) = ((unsigned long) address >> 48) & 0xffff;
+  *(short *) (function +22) = (unsigned long long) address & 0xffff;
+  *(int *)   (function +24) = ((unsigned long long) address >> 16) & 0xffffffff;
+  *(short *) (function +28) = ((unsigned long long) address >> 48) & 0xffff;
   *(short *) (function +30) = 0xE0FF;
 #define is_tramp(function)  \
   *(unsigned short *) (function + 0) == 0xB848 && \
@@ -1235,10 +1359,10 @@ __TR_function alloc_trampoline (__TR_function address, void* variable, void* dat
   *(unsigned short *) (function +20) == 0xB848 && \
   *(unsigned short *) (function +30) == 0xE0FF
 #define hilo(hiword,loword)  \
-  (((unsigned long) (hiword) << 32) | (unsigned long) (loword))
+  (((unsigned long long) (hiword) << 32) | (unsigned long long) (loword))
 #define himidlo(hishort,midword,loshort)  \
-  (((unsigned long) (hishort) << 48) | (unsigned long) (midword) << 16 \
-   | (unsigned long) (loshort))
+  (((unsigned long long) (hishort) << 48) | (unsigned long long) (midword) << 16 \
+   | (unsigned long long) (loshort))
 #define tramp_address(function)  \
   himidlo(*(unsigned short *) (function +28), \
           *(unsigned int *)   (function +24), \
@@ -1347,11 +1471,11 @@ __TR_function alloc_trampoline (__TR_function address, void* variable, void* dat
 #if !defined(CODE_EXECUTABLE)
 #if defined(EXECUTABLE_VIA_MALLOC_THEN_MPROTECT) || defined(EXECUTABLE_VIA_MMAP_THEN_MPROTECT)
   /* Call mprotect on the pages that contain the range. */
-  { unsigned long start_addr = (unsigned long) function;
-    unsigned long end_addr = (unsigned long) (function + TRAMP_LENGTH);
+  { uintptr_t start_addr = (uintptr_t) function;
+    uintptr_t end_addr = (uintptr_t) (function + TRAMP_LENGTH);
     start_addr = start_addr & -pagesize;
     end_addr = (end_addr + pagesize-1) & -pagesize;
-   {unsigned long len = end_addr - start_addr;
+   {uintptr_t len = end_addr - start_addr;
 #if defined(HAVE_MACH_VM)
     if (vm_protect(task_self(),start_addr,len,0,VM_PROT_READ|VM_PROT_WRITE|VM_PROT_EXECUTE) != KERN_SUCCESS)
 #else
@@ -1371,7 +1495,7 @@ __TR_function alloc_trampoline (__TR_function address, void* variable, void* dat
 #ifndef TRAMP_CODE_LENGTH
 #define TRAMP_CODE_LENGTH TRAMP_LENGTH
 #endif
-#if !(defined(__hppanew__) || defined(__powerpcaix__) || defined(__powerpc64aix__) || defined(__ia64__))
+#if !(defined(__hppanew__) || defined(__hppa64new__) || defined(__powerpcaix__) || defined(__powerpc64aix__) || defined(__ia64__))
   /* Only needed if we really set up machine instructions. */
 #ifdef __i386__
 #if defined(_WIN32)
@@ -1381,8 +1505,8 @@ __TR_function alloc_trampoline (__TR_function address, void* variable, void* dat
 #endif
 #ifdef __m68k__
 #if defined(__NetBSD__) && defined(__GNUC__)
-  { register unsigned long _beg __asm__ ("%a1") = (unsigned long) function_x;
-    register unsigned long _len __asm__ ("%d1") = TRAMP_CODE_LENGTH;
+  { register uintptr_t _beg __asm__ ("%a1") = (uintptr_t) function_x;
+    register uintptr_t _len __asm__ ("%d1") = TRAMP_CODE_LENGTH;
     __asm__ __volatile__ (
       "move%.l %#0x80000004,%/d0\n\t" /* CC_EXTPURGE | C_IPURGE */
       "trap #12"                      /* kernel call ‘cachectl’ */
@@ -1393,8 +1517,8 @@ __TR_function alloc_trampoline (__TR_function address, void* variable, void* dat
   }
 #endif
 #if defined(__linux__) && defined(__GNUC__)
-  { register unsigned long _beg __asm__ ("%d1") = (unsigned long) function_x;
-    register unsigned long _len __asm__ ("%d4") = TRAMP_CODE_LENGTH + 32;
+  { register uintptr_t _beg __asm__ ("%d1") = (uintptr_t) function_x;
+    register uintptr_t _len __asm__ ("%d4") = TRAMP_CODE_LENGTH + 32;
     __asm__ __volatile__ (
       "move%.l %#123,%/d0\n\t"
       "move%.l %#1,%/d2\n\t"
@@ -1435,7 +1559,7 @@ __TR_function alloc_trampoline (__TR_function address, void* variable, void* dat
 #ifdef __alpha__
   __TR_clear_cache();
 #endif
-#ifdef __hppa__
+#if defined(__hppa__) || defined(__hppa64__)
   /* This assumes that the trampoline fits in at most two cache lines. */
   __TR_clear_cache(function_x,function_x+TRAMP_CODE_LENGTH-1);
 #endif
@@ -1460,19 +1584,19 @@ __TR_function alloc_trampoline (__TR_function address, void* variable, void* dat
 #endif
 
   /* 5. Return. */
-  return (__TR_function) (function_x + TRAMP_BIAS);
+  return (trampoline_function_t) (function_x + TRAMP_BIAS);
 }
 
-void free_trampoline (__TR_function function)
+void free_trampoline (trampoline_function_t function)
 {
 #if TRAMP_BIAS
-  function = (__TR_function)((char*)function - TRAMP_BIAS);
+  function = (trampoline_function_t)((char*)function - TRAMP_BIAS);
 #endif
 #if !defined(CODE_EXECUTABLE) && !defined(EXECUTABLE_VIA_MALLOC_THEN_MPROTECT)
 #ifdef EXECUTABLE_VIA_MMAP_FILE_SHARED
   /* Find the writable address corresponding to the executable address. */
-  { unsigned long page_x = (unsigned long) function & -(long)pagesize;
-    function -= ((long*)page_x)[0];
+  { uintptr_t page_x = (uintptr_t) function & -(intptr_t)pagesize;
+    function -= ((intptr_t*)page_x)[0];
   }
 #endif
   *(char**)function = freelist; freelist = (char*)function;
@@ -1487,7 +1611,7 @@ int is_trampoline (void* function)
 #ifdef is_tramp
 #ifdef __hppanew__
   void* tramp_address = &tramp;
-  if (!(((long)function & 3) == (TRAMP_BIAS & 3))) return 0;
+  if (!(((uintptr_t)function & 3) == (TRAMP_BIAS & 3))) return 0;
 #endif
   return ((is_tramp(((char*)function - TRAMP_BIAS))) ? 1 : 0);
 #else
@@ -1495,25 +1619,25 @@ int is_trampoline (void* function)
 #endif
 }
 
-__TR_function trampoline_address (void* function)
+trampoline_function_t trampoline_address (trampoline_function_t function)
 {
 #ifdef tramp_address
-  return (__TR_function)(tramp_address(((char*)function - TRAMP_BIAS)));
+  return (trampoline_function_t)(tramp_address(((char*)function - TRAMP_BIAS)));
 #else
   abort();
 #endif
 }
 
-void* trampoline_variable (void* function)
+void** trampoline_variable (trampoline_function_t function)
 {
 #ifdef tramp_variable
-  return (void*)(tramp_variable(((char*)function - TRAMP_BIAS)));
+  return (void**)(tramp_variable(((char*)function - TRAMP_BIAS)));
 #else
   abort();
 #endif
 }
 
-void* trampoline_data (void* function)
+void* trampoline_data (trampoline_function_t function)
 {
 #ifdef tramp_data
   return (void*)(tramp_data(((char*)function - TRAMP_BIAS)));
